package xyz.karx.spirc;

import org.bukkit.Bukkit;
import org.bukkit.ChatColor;

import java.io.*;
import java.net.Socket;

public class Bot {
    private final String USERNAME;
    private final String HOST;
    private final String CHANNEL;
    private final int PORT;

    private Socket s;

    private BufferedWriter bw;
    private BufferedReader br;
    private Chat chat;

    private boolean hasRespondedToFirstPing = false;
    private boolean hasJoined = false;
    private boolean isRunning = false;

    public Bot(String USERNAME, String HOST, String CHANNEL, int PORT) {
        this.USERNAME = USERNAME;
        this.HOST = HOST;
        this.CHANNEL = CHANNEL;
        this.PORT = PORT;
    }

    private void init() {
        try {
            s = new Socket(HOST, PORT);

            bw = new BufferedWriter(new OutputStreamWriter(s.getOutputStream()));
            br = new BufferedReader(new InputStreamReader(s.getInputStream()));
            chat = new Chat(USERNAME, CHANNEL);

            chat.sendToServer(bw, "USER " + USERNAME + " * * :" + USERNAME);
            chat.sendToServer(bw, "NICK " + USERNAME);

        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void loop() {
        final String[] placeholder = {""};
        isRunning = true;

        Bukkit.getScheduler().runTaskTimer(Main.getPlugin(Main.class), () -> {
            try {
                if ((placeholder[0] = br.readLine()) != null && isRunning) {
                    String line = placeholder[0].replaceAll("\\x1f|\\x02|\\x12|\\x0f|\\x16|\\x03(?:\\d{1,2}(?:,\\d{1,2})?)?", "");

                    Main.getPlugin(Main.class).getLogger().info(line);

                    if (line.contains("PRIVMSG")) {
                        String user = line.substring(1, line.indexOf("!"));
                        String message = line.replaceAll("^[\\w\\W]+ :", "");

                        Bukkit.broadcastMessage(ChatColor.BLUE + "[IRC] <" + user + "> " + ChatColor.WHITE + message);
                    } else if (line.contains("PING")) {
                        chat.sendToServer(bw, line.replaceFirst("I", "O"));

                        hasRespondedToFirstPing = true;
                    }

                    // This will join the channel once and then not again
                    // Won't run until the bot has responded to the first ping (aka registered)
//                    if (hasRespondedToFirstPing && !hasJoined) {
                    if (line.contains("End of message of the day")) {
                        chat.sendToServer(bw, "MODE +B");
                        chat.sendToServer(bw, "JOIN #" + CHANNEL);
                    }
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }, 0, 0);
    }

    public void start() {
        init();
        loop();
    }

    public void stop() {
        try {
            chat.sendToChat(bw, "Server shutting down!");
            chat.sendToServer(bw, "PART #" + CHANNEL);
            isRunning = false;
            s.close();
            bw.close();
            br.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public void sendMessage(String nickname, String message) {
        if (isRunning) {
            chat.sendToChat(bw, "<" + nickname + "> " + message);
        }
    }
}
